@@ -528,9 +528,10 @@ gnc_dense_cal_set_month(GncDenseCal *dcal, GDateMonth mon)
 static void
 _gnc_dense_cal_set_month(GncDenseCal *dcal, GDateMonth mon, gboolean redraw)
 {
-    GTimer *t = g_timer_new();
+    GTimer *t;
     if (dcal->month == mon)
         return;
+    t = g_timer_new();
     dcal->month = mon;
     g_timer_start(t);
     recompute_first_of_month_offset(dcal);
@@ -818,8 +819,11 @@ gdc_free_all_mark_data(GncDenseCal *dcal)
     /* Remove the old mark data. */
     for (l = dcal->markData; l; l = l->next)
     {
-        g_list_free(((gdc_mark_data*)l->data)->ourMarks);
-        g_free((gdc_mark_data*)l->data);
+        gdc_mark_data *mark = l->data;
+        g_list_free (mark->ourMarks);
+        g_free (mark->name);
+        g_free (mark->info);
+        g_free (mark);
     }
     g_list_free(dcal->markData);
     dcal->markData = NULL;
@@ -902,13 +906,14 @@ gnc_dense_cal_draw_to_buffer(GncDenseCal *dcal)
     cairo_t *cr;
     gchar *primary_color_class, *secondary_color_class, *marker_color_class;
 
-    timer = g_timer_new();
     DEBUG("drawing");
     widget = GTK_WIDGET(dcal);
 
     if (!dcal->surface)
         return;
 
+    timer = g_timer_new();
+
     g_timer_start(timer);
     cr = cairo_create (dcal->surface);
     layout = gtk_widget_create_pango_layout(GTK_WIDGET(dcal), NULL);
@@ -1885,26 +1890,22 @@ cleanup:
 static void
 gdc_add_markings(GncDenseCal *cal)
 {
-    GList *tags;
-    tags = gnc_dense_cal_model_get_contained(cal->model);
-    for (; tags != NULL; tags = tags->next)
-    {
-        guint tag = GPOINTER_TO_UINT(tags->data);
-        gdc_add_tag_markings(cal, tag);
-    }
+    GList *tags = gnc_dense_cal_model_get_contained(cal->model);
+
+    for (GList *n = tags; n; n = n->next)
+        gdc_add_tag_markings(cal, GPOINTER_TO_UINT(n->data));
+
     g_list_free(tags);
 }
 
 static void
 gdc_remove_markings(GncDenseCal *cal)
 {
-    GList *tags;
-    tags = gnc_dense_cal_model_get_contained(cal->model);
-    for (; tags != NULL; tags = tags->next)
-    {
-        guint tag = GPOINTER_TO_UINT(tags->data);
-        gdc_mark_remove(cal, tag, FALSE);
-    }
+    GList *tags = gnc_dense_cal_model_get_contained(cal->model);
+
+    for (GList *n = tags; n; n = n->next)
+        gdc_mark_remove(cal, GPOINTER_TO_UINT(n->data), FALSE);
+
     g_list_free(tags);
 }
 
@@ -2048,6 +2049,8 @@ gdc_mark_remove(GncDenseCal *dcal, guint mark_to_remove, gboolean redraw)
     }
     g_list_free(mark_data->ourMarks);
     dcal->markData = g_list_remove(dcal->markData, mark_data);
+    g_free (mark_data->name);
+    g_free (mark_data->info);
     g_free(mark_data);
 
     if (redraw)
