@@ -63,6 +63,7 @@
 #include "gnc-ui-util.h"
 #include "gnc-tree-model-split-reg.h"
 #include "gnc-tree-control-split-reg.h"
+#include <gnc-glib-utils.h>
 
 #include "gnc-sx-instance-model.h"
 #include "dialog-sx-since-last-run.h"
@@ -836,7 +837,7 @@ gnc_sxed_check_consistent (GncSxEditorDialog2 *sxed)
 
         g_date_clear (&nextDate, 1);
         gnc_frequency_save_to_recurrence (sxed->gncfreq, &schedule, &startDate);
-        if (g_list_length (schedule) > 0)
+        if (gnc_list_length_cmp (schedule, 0))
         {
             g_date_subtract_days (&startDate, 1);
             recurrenceListNextInstance (schedule, &startDate, &nextDate);
@@ -1704,7 +1705,7 @@ _sx_engine_event_handler (QofInstance *ent, QofEventId event_type, gpointer user
     book = qof_instance_get_book (QOF_INSTANCE (acct));
     affected_sxes = gnc_sx_get_sxes_referencing_account (book, acct);
 
-    if (g_list_length (affected_sxes) == 0)
+    if (!gnc_list_length_cmp (affected_sxes, 0))
         return;
 
     {
