@@ -95,6 +95,9 @@ struct _transactioninfo
 
     /* Reference id to link gnc transaction to external object. E.g. aqbanking job id. */
     guint32 ref_id;
+
+    /* When updating a matched transaction, append Description and Notes instead of replacing */
+    gboolean append_text;
 };
 
 /* Some simple getters and setters for the above data types. */
@@ -240,6 +243,15 @@ gnc_import_TransInfo_set_ref_id (GNCImportTransInfo *info,
 }
 
 
+void
+gnc_import_TransInfo_set_append_text (GNCImportTransInfo *info,
+                                      gboolean append_text)
+{
+    g_assert (info);
+    info->append_text = append_text;
+}
+
+
 Split *
 gnc_import_MatchInfo_get_split (const GNCImportMatchInfo * info)
 {
@@ -449,7 +461,7 @@ TransactionGetTokens(GNCImportTransInfo *info)
     tokens = tokenize_string(tokens, text);
 
     /* The day of week the transaction occurred is a good indicator of
-     * what account this transaction belongs in.  Get the date and covert
+     * what account this transaction belongs in.  Get the date and convert
      * it to day of week as a token
      */
     transtime = xaccTransGetDate(transaction);
@@ -827,6 +839,93 @@ void split_find_match (GNCImportTransInfo * trans_info,
 /***********************************************************************
  */
 
+/* append the imported transaction description to the matched transaction description */
+static void
+desc_append (Transaction* selected_match_trans, gchar *new_desc)
+{
+    const gchar* curr_desc = xaccTransGetDescription (selected_match_trans);
+    gchar* tmp = g_strconcat(curr_desc, "|", new_desc, NULL);
+    xaccTransSetDescription (selected_match_trans, tmp);
+    g_free (tmp);
+}
+
+/* append the imported transaction notes to the matched transaction notes */
+static void
+notes_append (Transaction* selected_match_trans, gchar* new_notes)
+{
+    const gchar* curr_notes = xaccTransGetNotes (selected_match_trans);
+    gchar* tmp = g_strconcat (curr_notes, "|", new_notes, NULL);
+    xaccTransSetNotes (selected_match_trans, tmp );
+    g_free (tmp);
+}
+
+/* Append or replace transaction description and notes
+ * depending on the Append checkbox
+ */
+static void
+update_desc_and_notes (const GNCImportTransInfo* trans_info)
+{
+    GNCImportMatchInfo* selected_match =
+            gnc_import_TransInfo_get_selected_match (trans_info);
+    Transaction* imp_trans = gnc_import_TransInfo_get_trans (trans_info);
+
+    if (trans_info->append_text)
+    {
+        gchar *desc_imported, *desc_matched, *note_imported, *note_matched;
+        const gchar* raw_str = xaccTransGetDescription (imp_trans);
+
+        desc_imported =
+            raw_str ? g_utf8_normalize (raw_str, -1, G_NORMALIZE_ALL) : NULL;
+        raw_str = xaccTransGetDescription (selected_match->trans);
+        desc_matched =
+            raw_str ? g_utf8_normalize (raw_str, -1, G_NORMALIZE_ALL) : NULL;
+        raw_str = xaccTransGetNotes (imp_trans);
+        note_imported =
+            raw_str ? g_utf8_normalize (raw_str, -1, G_NORMALIZE_ALL) : NULL;
+        raw_str = xaccTransGetNotes (selected_match->trans);
+        note_matched =
+           raw_str ? g_utf8_normalize (raw_str, -1, G_NORMALIZE_ALL) : NULL;
+
+        // Append if desc_imported not already in desc_matched
+        if (desc_imported &&
+            (!desc_matched ||
+             g_utf8_strlen (desc_imported, -1) > g_utf8_strlen (desc_matched, -1) ||
+             !strstr (desc_matched, desc_imported)))
+        {
+            if (desc_matched && *desc_matched)
+                desc_append (selected_match->trans, desc_imported);
+            else
+                xaccTransSetDescription (selected_match->trans, desc_imported);
+        }
+
+        // Append if note_imported not already in note_matched
+        if (note_imported &&
+            (!note_matched ||
+             g_utf8_strlen (note_imported, -1) > g_utf8_strlen (note_matched, -1) ||
+             !strstr (note_matched, note_imported)))
+        {
+            if (note_matched && *note_matched)
+                notes_append (selected_match->trans, note_imported);
+            else
+                xaccTransSetNotes (selected_match->trans, note_imported);
+        }
+
+        g_free(desc_imported);
+        g_free(desc_matched);
+        g_free(note_imported);
+        g_free(note_matched);
+    }
+    else
+    {
+        // replace the matched transaction description with the imported transaction description
+        xaccTransSetDescription (selected_match->trans,
+                                 xaccTransGetDescription (imp_trans));
+        // replace the matched transaction notes with the imported transaction notes
+        xaccTransSetNotes (selected_match->trans,
+                           xaccTransGetNotes (imp_trans));
+    }
+}
+
 /** /brief -- Processes one match
    according to its selected action.  */
 gboolean
@@ -944,13 +1043,7 @@ gnc_import_process_trans_item (GncImportMatchMap *matchmap,
                    to balance the transaction */
             }
 
-            xaccTransSetDescription(selected_match->trans,
-                                    xaccTransGetDescription(
-                                        gnc_import_TransInfo_get_trans(trans_info)));
-
-            xaccTransSetNotes(selected_match->trans,
-                                    xaccTransGetNotes(
-                                        gnc_import_TransInfo_get_trans(trans_info)));
+            update_desc_and_notes( trans_info);
 
             if (xaccSplitGetReconcile(selected_match->split) == NREC)
             {
@@ -964,8 +1057,10 @@ gnc_import_process_trans_item (GncImportMatchMap *matchmap,
                the match will be remembered */
             if (gnc_import_split_has_online_id(trans_info->first_split))
             {
-                gnc_import_set_split_online_id(selected_match->split,
-                                               gnc_import_get_split_online_id(trans_info->first_split));
+                char *online_id = gnc_import_get_split_online_id
+                    (trans_info->first_split);
+                gnc_import_set_split_online_id(selected_match->split, online_id);
+                g_free (online_id);
             }
 
             /* Done editing. */
@@ -1019,9 +1114,12 @@ gnc_import_process_trans_item (GncImportMatchMap *matchmap,
             /* Copy the online id to the reconciled transaction, so
             		 the match will be remembered */
             if (gnc_import_split_has_online_id(trans_info->first_split))
-                gnc_import_set_split_online_id
-                (selected_match->split,
-                 gnc_import_get_split_online_id(trans_info->first_split));
+            {
+                char *online_id = gnc_import_get_split_online_id
+                    (trans_info->first_split);
+                gnc_import_set_split_online_id (selected_match->split, online_id);
+                g_free (online_id);
+            }
 
             /* Done editing. */
             /*DEBUG("CommitEdit selected_match")*/
@@ -1059,8 +1157,8 @@ static gint check_trans_online_id(Transaction *trans1, void *user_data)
     Account *account;
     Split *split1;
     Split *split2 = user_data;
-    const gchar *online_id1;
-    const gchar *online_id2;
+    gchar *online_id1, *online_id2;
+    gint retval;
 
     account = xaccSplitGetAccount(split2);
     split1 = xaccTransFindSplitByAccount(trans1, account);
@@ -1071,24 +1169,22 @@ static gint check_trans_online_id(Transaction *trans1, void *user_data)
        instead of the transactions */
     g_assert(split1 != NULL);
 
-    if (gnc_import_split_has_online_id(split1))
-        online_id1 = gnc_import_get_split_online_id(split1);
-    else
-        online_id1 = gnc_import_get_trans_online_id(trans1);
+    online_id1 = gnc_import_get_split_online_id (split1);
+
+    if (!online_id1 || !online_id1[0])
+    {
+        if (online_id1)
+            g_free (online_id1);
+        online_id1 = gnc_import_get_trans_online_id (trans1);
+    }
 
     online_id2 = gnc_import_get_split_online_id(split2);
 
-    if ((online_id1 == NULL) ||
-            (online_id2 == NULL) ||
-            (strcmp(online_id1, online_id2) != 0))
-    {
-        return 0;
-    }
-    else
-    {
-        /*printf("test_trans_online_id(): Duplicate found\n");*/
-        return 1;
-    }
+    retval = (!online_id1 || !online_id2 || strcmp (online_id1, online_id2)) ? 0 : 1;
+
+    g_free (online_id1);
+    g_free (online_id2);
+    return retval;
 }
 
 /** Checks whether the given transaction's online_id already exists in
@@ -1098,30 +1194,38 @@ gboolean gnc_import_exists_online_id (Transaction *trans, GHashTable* acct_id_ha
     gboolean online_id_exists = FALSE;
     Account *dest_acct;
     Split *source_split;
+    char *source_online_id;
 
     /* Look for an online_id in the first split */
     source_split = xaccTransGetSplit(trans, 0);
     g_assert(source_split);
 
+    source_online_id = gnc_import_get_split_online_id (source_split);
+
     // No online id, no point in continuing. We'd crash if we tried.
-    if (!gnc_import_get_split_online_id (source_split))
+    if (!source_online_id)
         return FALSE;
-    // Create a hash per account of a hash of all split IDs. Then the test below will be fast if
-    // we have many transactions to import.
+
+    // Create a hash per account of a hash of all split IDs. Then the
+    // test below will be fast if we have many transactions to import.
     dest_acct = xaccSplitGetAccount (source_split);
     if (!g_hash_table_contains (acct_id_hash, dest_acct))
     {
-        GHashTable* new_hash = g_hash_table_new (g_str_hash, g_str_equal);
-        GList* split_list = xaccAccountGetSplitList(dest_acct);
+        GHashTable* new_hash = g_hash_table_new_full
+            (g_str_hash, g_str_equal, g_free, NULL);
         g_hash_table_insert (acct_id_hash, dest_acct, new_hash);
-        for (;split_list;split_list=split_list->next)
+        for (GList *n = xaccAccountGetSplitList (dest_acct) ; n; n=n->next)
         {
-            if (gnc_import_split_has_online_id (split_list->data))
-                g_hash_table_add (new_hash, (void*) gnc_import_get_split_online_id (split_list->data));
+            if (gnc_import_split_has_online_id (n->data))
+            {
+                char *id = gnc_import_get_split_online_id (n->data);
+                if (!g_hash_table_insert (new_hash, (void*) id, GINT_TO_POINTER (1)))
+                    g_free (id);
+            }
         }
     }
     online_id_exists = g_hash_table_contains (g_hash_table_lookup (acct_id_hash, dest_acct),
-                                              gnc_import_get_split_online_id (source_split));
+                                              source_online_id);
     
     /* If it does, abort the process for this transaction, since it is
        already in the system. */
@@ -1131,6 +1235,7 @@ gboolean gnc_import_exists_online_id (Transaction *trans, GHashTable* acct_id_ha
         xaccTransDestroy(trans);
         xaccTransCommitEdit(trans);
     }
+    g_free (source_online_id);
     return online_id_exists;
 }
 
